# Contributing to PyTorch Geometric

If you are interested in contributing to PyTorch Geometric, your contributions will fall into two categories:

1. You want to implement a new feature:
    - In general, we accept any features as long as they fit the scope of this package. If you are unsure about this or need help on the design/implementation of your feature, post about it in an issue.
2. You want to fix a bug:
    - Feel free to send a Pull Request any time you encounter a bug. Please provide a clear and concise description of what the bug was. If you are unsure about if this is a bug at all or how to fix, post about it in an issue.

Once you finish implementing a feature or bug-fix, please send a Pull Request to https://github.com/rusty1s/pytorch_geometric.

## Developing PyTorch Geometric

To develop PyTorch Geometric on your machine, here are some tips:

1. Uninstall all existing PyTorch Geometric installs:

```
pip uninstall torch-geometric
pip uninstall torch-geometric  # run this command twice
```

2. Clone a copy of PyTorch Geometric from source:

```
git clone https://github.com/rusty1s/pytorch_geometric
cd pytorch_geometric
```

3. If you already have a PyTorch Geometric from source, update it:

```
git pull
```

4. Install PyTorch Geometric in `develop` mode:

```
python setup.py develop
```

This mode will symlink the Python files from the current local source tree into the Python install.
Hence, if you modify a Python file, you do not need to reinstall PyTorch Geometric again and again.

5. Ensure that you have a working PyTorch Geometric installation by running the entire test suite with

```
python setup.py test
```

In case an error occurs, please first check if all sub-packages ([`torch-scatter`](https://github.com/rusty1s/pytorch_scatter), [`torch-sparse`](https://github.com/rusty1s/pytorch_sparse), [`torch-cluster`](https://github.com/rusty1s/pytorch_cluster) and [`torch-spline-conv`](https://github.com/rusty1s/pytorch_spline_conv)) are on its latest reported version.

## Unit Testing

PyTorch Geometric's testing is located under `test/`.
Run the entire test suite with

```
python setup.py test
```

or run individual test files, like `pytest --no-cov test/utils/test_convert.py`, for individual test suites.

## Continuous Integration

PyTorch Geometric uses [Travis CI](https://travis-ci.org/rusty1s/pytorch_geometric) in combination with [CodeCov](https://codecov.io/github/rusty1s/pytorch_geometric?branch=master) for continuous integration.

Everytime you send a Pull Request, your commit will be built and checked against the PyTorch Geometric guidelines:

1. Ensure that your code is formatted correctly by testing against the styleguides of [`flake8`](https://github.com/PyCQA/flake8) and [`pycodestyle`](https://github.com/PyCQA/pycodestyle):

```
pycodestyle torch_geometric test examples
flake8 torch_geometric test examples
```

If you do not want to format your code manually, we recommend to use [`yapf`](https://github.com/google/yapf).

2. Ensure that the entire test suite passes and that code coverage roughly stays the same.
Please feel encouraged to provide a test with your submitted code.

```
python setup.py test
```

## Building Documentation

To build the documentation:

1. [Build and install](#developing-pytorch-geometric) PyTorch Geometric from source.
2. Install [Sphinx](https://www.sphinx-doc.org/en/master/) via `pip install sphinx sphinx_rtd_theme`.
3. Generate the documentation file via:
```
cd pytorch_geometric/docs
make html
```
The docs are located in `pytorch_geometric/docs/build/html/index.html`.
